unit uSingletonTest;

interface

uses
  TestFramework
  , uSimpleSingleton
  , uSimpleSingleton2
  , uSimpleSingleton3
  ;

type

  TSimpleSingletonTest = class(TTestCase)
  private

  published
    procedure TestNotNil;
    procedure TestCreateError;
    procedure TestFreeError;
  end;

  TSimpleSingleton2Test = class(TTestCase)
  private

  published
    procedure TestNotNil;
    procedure TestCreateError;
    procedure TestFreeError;
  end;

  TSimpleSingleton3Test = class(TTestCase)
  private

  published
    procedure TestNotNil;
    procedure TestCreateError;
    procedure TestFreeError;
  end;

implementation

uses
  sysutils                    // for Exception class
  ;

{ TSingletonTest }

procedure TSimpleSingletonTest.TestCreateError;
var
  ssTmp: TSimpleSingleton;
begin
  // See what happens when we call the Create method
  ssTmp := nil;
  ssTmp := TSimpleSingleton.Create;
  // Ideally, we don't want anything to happen
  Check(not Assigned(ssTmp), 'SimpleSingleton is assigned');
end;

procedure TSimpleSingletonTest.TestFreeError;
var
  ssTmp: TSimpleSingleton;
  nTmp: integer;
begin
// Uncomment this code with care - it thoroughly messes things up!
{  // See what happens when we call the singleton's free method
  ssTmp := nil;
  // Get a reference to the singleton
  ssTmp := SimpleSingleton;
  Check(Assigned(ssTmp), 'SimpleSingleton not assigned (1st time)');
  // Now call the free method
  ssTmp.Free;
  ssTmp := nil;
  // Now try getting another reference
  ssTmp := SimpleSingleton;
  // If we get this far, make sure we've got a valid reference
  Check(Assigned(ssTmp), 'SimpleSingleton not assigned (2nd time)');
  // Try to reference the singleton's property
  ssTmp.Value := 100;
  // Try to call the object's Create method
  ssTmp.Create;}
end;

procedure TSimpleSingletonTest.TestNotNil;
var
  ssTmp: TSimpleSingleton;
begin
  // We should be able to get the singleton from the function
  ssTmp := nil;
  ssTmp := SimpleSingleton;
  Check(Assigned(ssTmp), 'SimpleSingleton not assigned');
end;

{ TSimpleSingleton2Test }

procedure TSimpleSingleton2Test.TestCreateError;
var
  ssTmp: TSimpleSingleton2;
begin
  // See what happens when we call the Create method
  ssTmp := nil;
  ssTmp := TSimpleSingleton2.Create;
  // Ideally, we don't want anything to happen
  Check(not Assigned(ssTmp), 'SimpleSingleton2 is assigned');
end;

procedure TSimpleSingleton2Test.TestFreeError;
var
  ssTmp: TSimpleSingleton2;
  nTmp: integer;
begin
  // See what happens when we call the singleton's free method
  ssTmp := nil;
  // Get a reference to the singleton
  ssTmp := SimpleSingleton2;
  Check(Assigned(ssTmp), 'SimpleSingleton2 not assigned (1st time)');
  // Now call the free method
  ssTmp.Free;
  ssTmp := nil;
  // Now try getting another reference
  ssTmp := SimpleSingleton2;
  // If we get this far, make sure we've got a valid reference
  Check(Assigned(ssTmp), 'SimpleSingleton2 not assigned (2nd time)');
  // Try to reference the singleton's property
  ssTmp.Value := 100;
  // Try to call the object's Create method
  ssTmp.Create;
end;

procedure TSimpleSingleton2Test.TestNotNil;
var
  ssTmp: TSimpleSingleton2;
begin
  // We should be able to get the singleton from the function
  ssTmp := nil;
  ssTmp := SimpleSingleton2;
  Check(Assigned(ssTmp), 'SimpleSingleton2 not assigned');
end;

{ TSimpleSingleton3Test }

procedure TSimpleSingleton3Test.TestCreateError;
var
  ssTmp, ssTmp2: TSimpleSingleton3;
begin
  // See what happens when we call the Create method
  ssTmp := nil;
  ssTmp2 := nil;
  ssTmp := TSimpleSingleton3.Create;
  Check(ssTmp.InitCount = 1, 'InitCount is not 1');
  ssTmp2 := TSimpleSingleton3.Create;
  Check(ssTmp.InitCount = 1, 'InitCount is not 1');
  Check(Assigned(ssTmp), 'ssTmp not assigned');
  Check(Assigned(ssTmp2), 'ssTmp2 not assigned');
  Check(ssTmp = ssTmp2, 'Singleton references are not the same');
end;

procedure TSimpleSingleton3Test.TestFreeError;
var
  ssTmp, ssTmp2: TSimpleSingleton3;
  nTmp: integer;
begin
  // See what happens when we call the singleton's free method
  ssTmp := nil;
  // Get references to the singleton
  ssTmp := TSimpleSingleton3.Create;
  ssTmp2 := TSimpleSingleton3.Create;
  Check(Assigned(ssTmp), 'ssTmp not assigned');
  Check(Assigned(ssTmp2), 'ssTmp2 not assigned');
  Check(ssTmp = ssTmp2, 'Singleton references are not the same');
  // Now call the free method. Should get an EAbort exception
  try
    ssTmp.Free;
    fail('EAbort exception not raised');
  except
    on E: Exception do
    begin
      if E.ClassType <> EAbort then
        raise;
    end
  end;
  ssTmp := nil;
  // Try to reference the singleton's property
  ssTmp2.Value := 100;
  // Get another reference to the singleton
  ssTmp := TSimpleSingleton3.Create;
  // Check that we've still got the value
  Check(ssTmp.Value = 100, 'Value not retained');
end;

procedure TSimpleSingleton3Test.TestNotNil;
var
  ssTmp: TSimpleSingleton3;
begin
  // We should be able to get the singleton from the function
  ssTmp := nil;
  ssTmp := TSimpleSingleton3.Create;
  Check(Assigned(ssTmp), 'ssTmp not assigned');
end;

initialization
begin
  TestFramework.RegisterTest(TSimpleSingletonTest.Suite);
  TestFramework.RegisterTest(TSimpleSingleton2Test.Suite);
  TestFramework.RegisterTest(TSimpleSingleton3Test.Suite);
end;

end.
